@@ -117,7 +117,7 @@ click.rich_click.OPTION_GROUPS = {
     "genomad nn-classification": [
         {
             "name": "Basic options",
-            "options": ["--cleanup", "--restart", "--verbose"],
+            "options": ["--cleanup", "--restart", "--threads", "--verbose"],
         },
         {
             "name": "Advanced options",
@@ -652,6 +652,14 @@ def marker_classification(input, output, database, restart, threads, verbose):
     show_default=True,
     help="Overwrite existing intermediate files.",
 )
+@click.option(
+    "--threads",
+    "-t",
+    type=int,
+    default=multiprocessing.cpu_count(),
+    show_default=True,
+    help="Number of threads to use.",
+)
 @click.option(
     "--verbose/--quiet",
     "-v/-q",
@@ -685,14 +693,14 @@ def marker_classification(input, output, database, restart, threads, verbose):
             to reduce memory comsumption at the cost of speed.""",
 )
 def nn_classification(
-    input, output, single_window, batch_size, restart, verbose, cleanup
+    input, output, single_window, batch_size, restart, threads, verbose, cleanup
 ):
     """
     Classify the sequences in the [u]INPUT[/u] file (FASTA format) using the
     geNomad neural network and write the results to the [u]OUTPUT[/u] directory.
     """
     genomad.nn_classification.main(
-        input, output, single_window, batch_size, restart, verbose, cleanup
+        input, output, single_window, batch_size, restart, threads, verbose, cleanup
     )
 
 
@@ -801,9 +809,7 @@ def score_calibration(input, output, composition, force_auto, verbose):
     default=0.7,
     show_default=True,
     is_eager=True,
-    help="""Minimum score to flag a sequence as virus or plasmid. By default,
-            the sequence is classified as virus/plasmid if its virus/plasmid
-            score is higher than its chromosome score, regardless of the value.""",
+    help="""Minimum score to flag a sequence as virus or plasmid.""",
 )
 @click.option(
     "--max-fdr",
@@ -900,10 +906,11 @@ def summary(
     max_uscg,
 ):
     """
-    Generates a classification report file for the sequences in the [u]INPUT[/u]
-    file (FASTA format) and write it to the [u]OUTPUT[/u] directory. This module
-    requires that at least one of the base classification modules was executed
-    previously ([cyan]marker-classification[/cyan], [cyan]nn-classification[/cyan]).
+    Applies post-classification filters, generates classification reports for the
+    sequences in the [u]INPUT[/u] file (FASTA format), and writes them to the
+    [u]OUTPUT[/u] directory. This module requires that at least one of the base
+    classification modules was executed previously ([cyan]marker-classification[/cyan],
+    [cyan]nn-classification[/cyan]).
     """
     genomad.summary.main(
         input,
@@ -1051,9 +1058,7 @@ def summary(
     default=0.7,
     show_default=True,
     is_eager=True,
-    help="""Minimum score to flag a sequence as virus or plasmid. By default,
-            the sequence is classified as virus/plasmid if its virus/plasmid
-            score is higher than its chromosome score, regardless of the value.""",
+    help="""Minimum score to flag a sequence as virus or plasmid.""",
 )
 @click.option(
     "--max-fdr",
@@ -1247,6 +1252,7 @@ def end_to_end(
             cleanup=cleanup,
             verbose=verbose,
             restart=restart,
+            threads=threads,
         )
         ctx.invoke(
             aggregated_classification,
