@@ -19,6 +19,8 @@ FEATURE_FILE_HEADER = "\t".join(
         "seq_name",
         "n_genes",
         "n_uscg",
+        "n_plasmid_hallmarks",
+        "n_virus_hallmarks",
         "genetic_code",
         "strand_switch_rate",
         "coding_density",
@@ -45,6 +47,9 @@ FEATURE_FILE_HEADER = "\t".join(
         "v_vs_p_score_logistic",
         "p_vs_c_score_logistic",
         "gv_marker_freq",
+        "marker_enrichment_c",
+        "marker_enrichment_p",
+        "marker_enrichment_v",
     ]
 )
 
@@ -56,6 +61,8 @@ class AnnotatedContig:
     coding_length: int = 0
     n_genes: int = 0
     n_usgc: int = 0
+    n_plasmid_hallmarks: int = 0
+    n_virus_hallmarks: int = 0
     genetic_code: int = 11
     n_cc_markers: int = 0
     n_cp_markers: int = 0
@@ -115,6 +122,20 @@ class AnnotatedContig:
             score = (np.exp(self.spm_p) - np.exp(self.spm_c)).sum()
         return score
 
+    def get_marker_enrichment(self, score_type: str) -> float:
+        possible_score_types = {"c", "p", "v"}
+        if score_type not in possible_score_types:
+            raise ValueError(
+                f"Invalid score type. Expected one of: {possible_score_types}"
+            )
+        if score_type == "c":
+            score = (np.exp(self.spm_c) - np.exp(np.add(self.spm_p, self.spm_v))).sum()
+        elif score_type == "p":
+            score = (np.exp(self.spm_p) - np.exp(np.add(self.spm_c, self.spm_v))).sum()
+        elif score_type == "v":
+            score = (np.exp(self.spm_v) - np.exp(np.add(self.spm_c, self.spm_p))).sum()
+        return score
+
 
 def yield_annotated_contigs(
     input_path: Path,
@@ -123,7 +144,7 @@ def yield_annotated_contigs(
     rbs_categories_dict: dict,
 ) -> AnnotatedContig:
     annotated_contigs_dict = {
-        seq.id: AnnotatedContig(seq.id, len(seq))
+        seq.accession: AnnotatedContig(seq.accession, len(seq))
         for seq in sequence.read_fasta(input_path, strip_n=True)
     }
     marker_features_dict = database_obj.get_marker_features()
@@ -153,7 +174,9 @@ def yield_annotated_contigs(
             spm_v,
             gv_marker,
             uscg,
-        ) = marker_features_dict.get(match, (None, 0.0, 0.0, 0.0, 0, 0))
+            plasmid_hallmark,
+            virus_hallmark,
+        ) = marker_features_dict.get(match, (None, 0.0, 0.0, 0.0, 0, 0, 0, 0))
         # Contigs that only contain Ns won't be in `annotated_contigs_dict`
         if contig in annotated_contigs_dict:
             annotated_contigs_dict[contig].n_genes += 1
@@ -167,6 +190,8 @@ def yield_annotated_contigs(
                 annotated_contigs_dict[contig].spm_v.append(spm_v)
                 annotated_contigs_dict[contig].n_gv_markers += gv_marker
                 annotated_contigs_dict[contig].n_usgc += uscg
+                annotated_contigs_dict[contig].n_plasmid_hallmarks += plasmid_hallmark
+                annotated_contigs_dict[contig].n_virus_hallmarks += virus_hallmark
                 if specificity_class == "CC":
                     annotated_contigs_dict[contig].n_cc_markers += 1
                 elif specificity_class == "CP":
@@ -208,8 +233,10 @@ def get_feature_array(
     contig_array = []
     n_genes_array = []
     n_uscg_array = []
+    n_hallmarks_array = []
     genetic_code_array = []
     feature_array = []
+    marker_enrichment_array = []
 
     rbs_categories_dict = {}
     for line in utils.read_file(rbs_file):
@@ -226,6 +253,7 @@ def get_feature_array(
         }
         n_genes_array.append(annotated_contig.n_genes)
         n_uscg_array.append(annotated_contig.n_usgc)
+        n_hallmarks_array.append([annotated_contig.n_plasmid_hallmarks, annotated_contig.n_virus_hallmarks])
         genetic_code_array.append(annotated_contig.genetic_code)
         contig_features = [
             annotated_contig.strand_switch_rate,
@@ -287,12 +315,20 @@ def get_feature_array(
             else 0.0,
         ]
         feature_array.append(contig_features)
+        marker_enrichment = [
+            annotated_contig.get_marker_enrichment("c"),
+            annotated_contig.get_marker_enrichment("p"),
+            annotated_contig.get_marker_enrichment("v"),
+        ]
+        marker_enrichment_array.append(marker_enrichment)
     return (
         np.array(contig_array),
         np.array(n_genes_array),
         np.array(n_uscg_array),
+        np.array(n_hallmarks_array),
         np.array(genetic_code_array),
         np.array(feature_array),
+        np.array(marker_enrichment_array),
     )
 
 
@@ -443,18 +479,24 @@ def main(input_path, output_path, database_path, restart, threads, verbose):
         contig_names = np.load(outputs.features_npz_output)["contig_names"]
         contig_n_genes = np.load(outputs.features_npz_output)["contig_n_genes"]
         contig_n_uscg = np.load(outputs.features_npz_output)["contig_n_uscg"]
+        contig_n_hallmarks = np.load(outputs.features_npz_output)["contig_n_hallmarks"]
         contig_genetic_code = np.load(outputs.features_npz_output)[
             "contig_genetic_code"
         ]
         contig_features = np.load(outputs.features_npz_output)["contig_features"]
+        contig_marker_enrichment = np.load(outputs.features_npz_output)[
+            "contig_marker_enrichment"
+        ]
     else:
         with console.status("Computing sequence features."):
             (
                 contig_names,
                 contig_n_genes,
                 contig_n_uscg,
+                contig_n_hallmarks,
                 contig_genetic_code,
                 contig_features,
+                contig_marker_enrichment,
             ) = get_feature_array(
                 input_path,
                 outputs.annotate_genes_output,
@@ -471,8 +513,10 @@ def main(input_path, output_path, database_path, restart, threads, verbose):
                 contig_names=contig_names,
                 contig_n_genes=contig_n_genes,
                 contig_n_uscg=contig_n_uscg,
+                contig_n_hallmarks=contig_n_hallmarks,
                 contig_genetic_code=contig_genetic_code,
                 contig_features=contig_features,
+                contig_marker_enrichment=contig_marker_enrichment,
             )
             console.log(
                 "Sequence features in binary format written to [green]"
@@ -486,15 +530,31 @@ def main(input_path, output_path, database_path, restart, threads, verbose):
     ):
         with open(outputs.features_output, "w") as fout:
             fout.write(f"{FEATURE_FILE_HEADER}\n")
-            for name, n_genes, n_uscg, genetic_code, features in zip(
+            for (
+                name,
+                n_genes,
+                n_uscg,
+                n_hallmarks,
+                genetic_code,
+                features,
+                marker_enrichment,
+            ) in zip(
                 contig_names,
                 contig_n_genes,
                 contig_n_uscg,
+                contig_n_hallmarks,
                 contig_genetic_code,
                 contig_features,
+                contig_marker_enrichment,
             ):
                 features = "".join(map(lambda x: f"{x:.4f}\t", features)).strip()
-                fout.write(f"{name}\t{n_genes}\t{n_uscg}\t{genetic_code}\t{features}\n")
+                marker_enrichment = "".join(
+                    map(lambda x: f"{x:.4f}\t", marker_enrichment)
+                ).strip()
+                fout.write(
+                    f"{name}\t{n_genes}\t{n_uscg}\t{n_hallmarks[0]}\t{n_hallmarks[1]}\t"
+                    f"{genetic_code}\t{features}\t{marker_enrichment}\n"
+                )
         console.log(
             "Sequence features in tabular format written to [green]"
             f"{outputs.features_output.name}[/green]."
@@ -513,20 +573,26 @@ def main(input_path, output_path, database_path, restart, threads, verbose):
         provirus_n_uscg = np.load(outputs.provirus_features_npz_output)[
             "provirus_n_uscg"
         ]
+        provirus_n_hallmarks = np.load(outputs.provirus_features_npz_output)["provirus_n_hallmarks"]
         provirus_genetic_code = np.load(outputs.provirus_features_npz_output)[
             "provirus_genetic_code"
         ]
         provirus_features = np.load(outputs.provirus_features_npz_output)[
             "provirus_features"
         ]
+        provirus_marker_enrichment = np.load(outputs.provirus_features_npz_output)[
+            "provirus_marker_enrichment"
+        ]
     elif classify_proviruses:
         with console.status("Computing provirus features."):
             (
                 provirus_names,
                 provirus_n_genes,
                 provirus_n_uscg,
+                provirus_n_hallmarks,
                 provirus_genetic_code,
                 provirus_features,
+                provirus_marker_enrichment,
             ) = get_feature_array(
                 outputs.find_proviruses_nucleotide_output,
                 outputs.find_proviruses_genes_output,
@@ -543,8 +609,10 @@ def main(input_path, output_path, database_path, restart, threads, verbose):
                 provirus_names=provirus_names,
                 provirus_n_genes=provirus_n_genes,
                 provirus_n_uscg=provirus_n_uscg,
+                provirus_n_hallmarks=provirus_n_hallmarks,
                 provirus_genetic_code=provirus_genetic_code,
                 provirus_features=provirus_features,
+                provirus_marker_enrichment=provirus_marker_enrichment,
             )
             console.log(
                 "Provirus features in binary format written to [green]"
@@ -559,16 +627,30 @@ def main(input_path, output_path, database_path, restart, threads, verbose):
         ):
             with open(outputs.provirus_features_output, "w") as fout:
                 fout.write(f"{FEATURE_FILE_HEADER}\n")
-                for name, n_genes, n_uscg, genetic_code, features in zip(
+                for (
+                    name,
+                    n_genes,
+                    n_uscg,
+                    n_hallmarks,
+                    genetic_code,
+                    features,
+                    marker_enrichment,
+                ) in zip(
                     provirus_names,
                     provirus_n_genes,
                     provirus_n_uscg,
+                    provirus_n_hallmarks,
                     provirus_genetic_code,
                     provirus_features,
+                    provirus_marker_enrichment,
                 ):
                     features = "".join(map(lambda x: f"{x:.4f}\t", features)).strip()
+                    marker_enrichment = "".join(
+                        map(lambda x: f"{x:.4f}\t", marker_enrichment)
+                    ).strip()
                     fout.write(
-                        f"{name}\t{n_genes}\t{n_uscg}\t{genetic_code}\t{features}\n"
+                        f"{name}\t{n_genes}\t{n_uscg}\t{n_hallmarks[0]}\t{n_hallmarks[1]}\t"
+                        f"{genetic_code}\t{features}\t{marker_enrichment}\n"
                     )
             console.log(
                 "Provirus features in tabular format written to [green]"
@@ -589,10 +671,9 @@ def main(input_path, output_path, database_path, restart, threads, verbose):
             console.error("No sequences were found. Please check your input FASTA.")
             sys.exit(1)
         with console.status("Classifying sequences."):
-            with utils.suppress_stdout():
-                df_model = xgb.Booster(
-                    {"nthread": threads}, model_file=GenomadData.decision_forest_file
-                )
+            df_model = xgb.Booster(
+                {"nthread": threads}, model_file=GenomadData.decision_forest_file
+            )
             contig_predictions = utils.softmax(
                 df_model.predict(xgb.DMatrix(contig_features), output_margin=True),
                 temperature=2,
@@ -642,10 +723,9 @@ def main(input_path, output_path, database_path, restart, threads, verbose):
         )["predictions"]
     elif classify_proviruses:
         with console.status("Classifying proviruses."):
-            with utils.suppress_stdout():
-                df_model = xgb.Booster(
-                    {"nthread": threads}, model_file=GenomadData.decision_forest_file
-                )
+            df_model = xgb.Booster(
+                {"nthread": threads}, model_file=GenomadData.decision_forest_file
+            )
             provirus_predictions = utils.softmax(
                 df_model.predict(xgb.DMatrix(provirus_features), output_margin=True),
                 temperature=2,
